﻿/*
  wavp_pwm.c - wavp library
  Copyright (c) 2015 Mitsuhiro Matsuura.  All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/
/*
  Modified 20 Mar 2017 by Nozomu Fujita
   ・PWM 再生周波数をライブラリに通知する wavp_pwmsf() 呼び出しを追加
*/

#include "iodefine.h"
#include "iodefine_ext.h"

#include "wavp_pwm.h"

#define TAU_CLOCK 32000000

void wavp_pwm_init(unsigned long sf)
{
	wavp_pwmsf(sf);
	wavp_pwmclk(TAU_CLOCK);

	PER0.BIT.tau0en	= 1;		// タイマ・アレイ・ユニット0イネーブル
	TPS0.tps0		&= ~0x0f;		// クロック選択	32MHz

	TT0.tt0			&= ~0x0007;	// タイマ・チャネル停止レジスタ0，チャネル0,1,2停止

	TMMK00			= 1U;		// 割り込みマスク
	TMIF00			= 0U;		// 割り込み要求フラグクリア

	TMMK01			= 1U;		// 割り込みマスク
	TMIF01			= 0U;		// 割り込み要求フラグクリア

	TMMK02			= 1U;		// 割り込みマスク
	TMIF02			= 0U;		// 割り込み要求フラグクリア

	// 割り込み優先順位レベル
	TMPR100			= 1U;
	TMPR000			= 0U;

	// チャネル0 PWMマスタ
	TMR00.tmr00		= 0x0001;			// インターバル・タイマ・モード，カウント開始時にタイマ割込みを発生させる
	TDR00.tdr00		= TAU_CLOCK/sf-1;	// WAVデータのサンプリング周波数にセット

	TOE0.BIT.bit0	= 0;				// タイマ出力許可レジスタ00，出力禁止

	// チャネル1 PWMスレーブ
	TMR01.tmr01		= 0x0409;			// ワンカウント・モード（PWMスレーブ）
	TDR01.tdr01		= TAU_CLOCK/sf/2-1;	// 中点出力

	TOM0.BIT.bit1	= 1;				// タイマ出力モード・レジスタ01，スレーブ・チャネル出力モード
	TOL0.BIT.bit1	= 0;				// タイマ出力レベル・レジスタ01，正論理出力

	TO0.BIT.bit1	= 0;				// タイマ出力レジスタ01，出力値0
	TOE0.BIT.bit1	= 1;				// タイマ出力許可レジスタ01，出力許可

	// チャネル2 PWMスレーブ
	TMR02.tmr02		= 0x409;			// ワンカウント・モード（PWMスレーブ）
	TDR02.tdr02		= TAU_CLOCK/sf/2-1;	// 中点出力

	TOM0.BIT.bit2	= 1;				// タイマ出力モード・レジスタ02，スレーブ・チャネル出力モード
	TOL0.BIT.bit2	= 0;				// タイマ出力レベル・レジスタ02，正論理出力

	TO0.BIT.bit2	= 0;				// タイマ出力レジスタ02，出力値0
	TOE0.BIT.bit2	= 1;				// タイマ出力許可レジスタ02，出力許可

	// タイマ・アレイ・ユニット動作開始
	TMIF00			= 0U;				// 割り込み要求フラグクリア
	TMMK00			= 0U;				// 割り込みマスククリア
	TS0.ts0			|= (4|2|1);			// タイマ・チャネル開始レジスタ0，TS02,TS01,TS00開始

	return;
}
